#!/usr/bin/python

import socket
from scapy.all import *
import threading

# 本地监听地址
localListenAddr = '0.0.0.0'
# 本地监听端口列表
localPorts = [38850, 61375, 61735]

# 目标地址（填你电脑内网地址）
sendDstAddr = '192.168.2.55'

# 上行接口名
sendSrcIface = "wg0"

# ==========

bufferSize = 5000

# 包计数器
packet_counter = 0
# 包计数器最大值
MAX_PACKET_COUNTER = 1_000_000

print("开始监听...")

# 创建一个锁对象
lock = threading.Lock()

def listen_on_port(port):
    global packet_counter
    try:
        # 创建一个UDP套接字
        UDPServerSocket = socket.socket(family=socket.AF_INET, type=socket.SOCK_DGRAM)
        UDPServerSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        UDPServerSocket.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        UDPServerSocket.bind((localListenAddr, port))
        print(f"成功绑定到端口 {port}")
    except OSError as e:
        print(f"无法绑定到端口 {port}: {e}")
        return

    while True:
        try:
            print("========================================")
            # 接收来自客户端的数据包
            bytesAddressPair = UDPServerSocket.recvfrom(bufferSize)
            message = bytesAddressPair[0]
            address = bytesAddressPair[1]
            
            with lock:
                packet_counter += 1
                if packet_counter > MAX_PACKET_COUNTER:
                    packet_counter = 1
                packet_id = packet_counter
                
                # 获取设备的源 MAC 地址
                src_mac = get_if_hwaddr(sendSrcIface)
                
                # 根据目标地址类型选择合适的 IP 层
                if ':' in sendDstAddr:
                    ip_layer = IPv6(dst=sendDstAddr, src=address[0])
                else:
                    ip_layer = IP(dst=sendDstAddr, src=address[0])
                
                # 构建新的IP和UDP包
                udp = UDP(sport=address[1], dport=address[1])  # 使用源数据包的目标端口
                payload = message
                packet = ip_layer / udp / payload
                
                # 打印详细的包信息
                print(f"[{packet_id}] 监听到数据包：")
                print(f"[{packet_id}] 源IP地址：{packet[IP].src}")
                print(f"[{packet_id}] 源端口：{packet[UDP].sport}")
                print(f"[{packet_id}] 目标IP地址：{packet[IP].dst}")
                print(f"[{packet_id}] 目标端口：{packet[UDP].dport}")
                print(f"[{packet_id}] 数据内容：{message}")
                print(f"[{packet_id}] 设备的源MAC地址：{src_mac}")

                # 发送数据包
                print(f"[{packet_id}] 正在将数据包发送到目标地址: {sendDstAddr}，目标端口: {address[1]}")
                try:
                    sendp(packet, iface=sendSrcIface)  # 使用设备的源MAC地址发送二层数据包
                    print(f"[{packet_id}] 发送二层数据包成功")
                except Exception as e:
                    print(f"[{packet_id}] 发送二层数据包失败: {str(e)}")
                
                try:
                    send(packet)  # 发送三层数据包
                    print(f"[{packet_id}] 发送三层数据包成功")
                except Exception as e:
                    print(f"[{packet_id}] 发送三层数据包失败: {str(e)}")

        except Exception as e:
            with lock:
                packet_counter += 1
                if packet_counter > MAX_PACKET_COUNTER:
                    packet_counter = 1
                packet_id = packet_counter
                
                print(f"[{packet_id}] 发生其他错误: {str(e)}")

# 为每个端口启动一个线程
threads = []
for port in localPorts:
    thread = threading.Thread(target=listen_on_port, args=(port,))
    thread.start()
    threads.append(thread)

# 等待所有线程完成
for thread in threads:
    thread.join()
            



## 注意，如果你想要以原设备的MAC地址发送数据包，需要先获取原设备的MAC地址，这边为了避免影响性能，这个版本没用sniff抓全包，只正常监听了udp，你可以手动往里头改mac，或者改回sniff。
# try:
#     # 获取捕获到的源设备的MAC地址
#     captured_src_mac = getmacbyip(address[0])
#     if captured_src_mac is None:
#         raise ValueError(f"无法获取IP地址 {address[0]} 的MAC地址")
#     sendp(Ether(src=captured_src_mac) / packet, iface=sendSrcIface)  # 使用捕获到的源设备的MAC地址发送二层数据包
#     print(f"[{packet_id}] 使用捕获到的源设备的MAC地址发送二层数据包成功")
# except Exception as e:
#     print(f"[{packet_id}] 使用捕获到的源设备的MAC地址发送二层数据包失败: {str(e)}")
